/*
 *     CodesPane.java  Apr 19, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.gui;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.Point;
import java.util.*;

public class CodesPane extends JTabbedPane implements ChangeListener, MouseListener {
    private MainFrame frame;
    private FileManager fileManager;
    private MenuPane menuPane;
    protected JPopupMenu popupMenu;
    protected CutCopyPasteMenu cppMenu;

    private ArrayList codes = new ArrayList();  // codePane;
    private int selectedIndex = 0;

    public CodesPane(MainFrame frame) {
        super(JTabbedPane.TOP);

        this.frame = frame;
        fileManager = frame.getFileManager();
        menuPane = frame.getMenuPane();
        
        popupMenu = new JPopupMenu("CutCopyPaste");
        cppMenu = new CutCopyPasteMenu(popupMenu, fileManager);
        popupMenu.addPopupMenuListener(cppMenu);

        addChangeListener(this);
        addMouseListener(this);

        init();
    }

    public MainFrame getFrame() {
        return frame;
    }

    public JPopupMenu getJPopupMenu() {
        return popupMenu;
    }

    private void init() { 
        CodeUndoManager undoManager = frame.createCodeUndoManager();
        CodePane codePane = new CodePane(this, undoManager);

        addTab(codePane.getTabName(), null, codePane, codePane.getFileName());
        setSelectedComponent(codePane);
        codes.add(codePane);
    }

    public CodePane add(String name, String content) {
        CodeUndoManager undoManager = frame.createCodeUndoManager();
        CodePane codePane = new CodePane(this, name, undoManager);
        codePane.setText(content);

        addTab(codePane.getTabName(), null, codePane, codePane.getFileName());
        setSelectedComponent(codePane);
        codes.add(codePane);

        CodePane firstPane = (CodePane)codes.get(0);
        if (firstPane.isNoName() && !firstPane.hasChanged()) {
            closeTab(0);
        }

        return codePane;
    }

    public boolean hasOpened(String name) {
        return getCodePane(name) != null;
    }

    public boolean hasChanged(String name) {
        CodePane codePane = getCodePane(name);

        return codePane != null && codePane.hasChanged();
    }

    public CodePane getCodePane(String name) {
        Iterator it = codes.iterator();
        while (it.hasNext()) {
            CodePane codePane = (CodePane)it.next();
            if (name.equals(codePane.getFileName())) {
                return codePane;
            }
        }
        return null;
    }

    public CodePane getCurrentCodePane() {
        if (getOpenFileNum() != 0 && selectedIndex < getOpenFileNum()) {
            CodePane codePane = (CodePane)codes.get(selectedIndex);
            return codePane;
        }
        return null;
    }

    public int getOpenFileNum() {
        return codes.size();
    }

    public void stateChanged(ChangeEvent evt) {
        JTabbedPane src = (JTabbedPane)evt.getSource();
        selectedIndex = src.getSelectedIndex();
        updateUndoState();
    }

    public ArrayList getCodes() {
        return codes;
    }

    public void updateTab() {
        CodePane codePane = getCurrentCodePane();
        setTitleAt(selectedIndex, codePane.getTabName());
        setToolTipTextAt(selectedIndex, codePane.getFileName());
    }

    public void closeTab() {
        closeTab(selectedIndex);
    }

    private void closeTab(int index) {
        codes.remove(index);
        remove(index);
    }

    public void mousePressed(MouseEvent evt) {
        showPopup(evt);
    }

    public void mouseReleased (MouseEvent evt) {
        showPopup(evt);
    }

    public void mouseClicked (MouseEvent evt) {
        showPopup(evt);
    }

    public void mouseEntered(MouseEvent evt) {
    }

    public void mouseExited(MouseEvent evt) {
    }
    
    private void showPopup(MouseEvent evt) {
        if (isPopupTrigger(evt)) {  // evt.isPopupTrigger()
            Point point = evt.getPoint();

            CodePane codePane = getCurrentCodePane();
            codePane.showCursor(point);
            cppMenu.updateMenuState();

            popupMenu.show(evt.getComponent(), point.x, point.y + 10);
        }
    }
    
    private boolean isPopupTrigger(MouseEvent evt) {
        return evt.getButton() == MouseEvent.BUTTON2 ||
               evt.getButton() == MouseEvent.BUTTON3;
    }

    public void updateIconState(CodePane codePane) {
        menuPane.updateIconState(codePane);
    }

    public void updateUndoState() {
        CodePane codePane = getCurrentCodePane();
        if (codePane != null) {
            menuPane.updateUndoState(codePane);
            menuPane.updateRedoState(codePane);
        }
    }
}
