/*
 *     JavaField.java  Apr 19, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.w3c.dom.*;
import java.util.*;

/*
  An object of this class will be never cached.
*/
public class JavaField extends JavaStatement {
    private JavaVariable jv;

    protected JavaField() {
    }

    public JavaField(Node node) {
        super(node);

        Node n = getFirstChildNode(elem, JXConstants.ExpressionElem);
        if (n.getNodeName().equals(JXConstants.ExpressionElem)) {
            jv = new JavaVariable(n, elem);
            removeDefVariable(jv);
            addDefVariableFirst(jv);
        }
    }

    public String getName() {
        return jv.getName();
    }

    public String getType() {
        return getElemType(elem);
    }

    public String getTypeFqn() {
        return getElemTypeFqn(elem);
    }

    public String getElemId() {
        return getElemId(elem);
    }

    public JavaClass getJavaClass() {
        return getJavaClass(elem);
    }

    public String getAccess() {
        String access = elem.getAttribute(JXConstants.AccessAttr);
        if (access.length() == 0) {
            access = JXConstants.Default;
        }
        return access;
    }

    public ArrayList getMethodCallNodes() {
        ArrayList mcalls = getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.MethodCall);
        mcalls.addAll(getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.ConstructorCall));
        mcalls.addAll(getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.SuperConstructorCall));
        return mcalls;
    }

    public JavaClassList getUsedTypes() {
        JavaClassList classes = new JavaClassList();

        Iterator it = getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaType jt = new JavaType(elem);
            JavaClass jc = jt.getReferredJavaClass();
            if (jc != null) {
                classes.add(jc);
            }
        }
        return classes;
    }

    public boolean equals(JavaField jf) {
        return jf != null && isInTheSameClass(jf) && getName().equals(jf.getName());
    }

    public boolean isPublic() {
        if (getJavaClass() != null && getJavaClass().isInterface()) {
            return true;
        }
        return getAccess().equals(JXConstants.Public);
    }

    public boolean isProtected() {
        return getAccess().equals(JXConstants.Protected);
    }

    public boolean isPrivate() {
        return getAccess().equals(JXConstants.Private);
    }

    public boolean isDefault() {
        return getAccess().equals(JXConstants.Default);
    }

    public boolean isFinal() {
        if (getJavaClass() != null && getJavaClass().isInterface()) {
            return true;
        }
        return elem.getAttribute(JXConstants.FinalAttr).equals(JXConstants.YES);
    }

    public boolean isStatic() {
        if (getJavaClass() != null && getJavaClass().isInterface()) {
            return true;
        }
        return elem.getAttribute(JXConstants.StaticAttr).equals(JXConstants.YES);
    }

    public boolean isVolatile() {
        return elem.getAttribute(JXConstants.VolatileAttr).equals(JXConstants.YES);
    }

    public boolean isTransient() {
        return elem.getAttribute(JXConstants.TransientAttr).equals(JXConstants.YES);
    }

    public boolean isOmitted() {
        return elem.getAttribute(JXConstants.OmittedAttr).equals(JXConstants.YES);
    }

    public boolean hasDefined() {
        return elem.getAttribute(JXConstants.WriteAttr).equals(JXConstants.YES);
    }
        
    public String getModifier() {
        StringBuffer buf = new StringBuffer();

        if (isPublic()) {
            buf.append(" public");
        } else if (isProtected()) {
            buf.append(" protected");
        } else if (isPrivate()) {
            buf.append(" private");
        }

        if (isFinal()) {
            buf.append(" final");
        }
        if (isStatic()) {
            buf.append(" static");
        }
        if (isVolatile()) {
            buf.append(" volatile");
        }
        if (isTransient()) {
            buf.append(" transient");
        }

        if (buf.length() != 0) {
            return buf.substring(1);
        }
        return "";
    }

    public JavaVariable getJavaVariable() {
        return jv;
    }

    public Node getIdentNode() {
        return getFirstChildNode(jv.getElem(), JXConstants.IdentElem);
    }

    public int getNumFieldInDeclaration() {
        String typeId = getTypeNode(elem).getAttribute(JXConstants.IdAttr);

        int num = 0;
        Iterator it = getJavaClass().getFieldNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            if (e.getAttribute(JXConstants.TypeFirstAttr).equals(typeId)) {
                num++;
            }
        }
        return num;
    }

    public JavaField getFieldWithDeclaration() {
        String typeId = getTypeNode(elem).getAttribute(JXConstants.IdAttr);

        Iterator it = getJavaClass().getFieldNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            if (e.getAttribute(JXConstants.TypeFirstAttr).equals(typeId) &&
                !e.getAttribute(JXConstants.OmittedAttr).equals(JXConstants.YES)) {
                return new JavaField(e);
            }
        }
        return null;
    }

    public void print(StringBuffer buf) {
        print("", buf);
    }

    public void print(String indent, StringBuffer buf) {
        buf.append(indent + "FIELD: " + getAccess() + " " + getName());
        buf.append("@" + getType() + "[" + getTypeFqn() + "]");
        buf.append(" = " + uses.toString() + "\n");
    }
}
