/*
 *     JavaMethod.java  Apr 19, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.j2xml.Repository;
import org.w3c.dom.*;
import java.util.*;

public class JavaMethod extends JavaElement {

    protected JavaMethod() {
    }        

    public JavaMethod(Node node) {
        super(node);
    }

    public String getName() {
        return getIdentName(elem);
    }
    
    public String getType() {
        if (isConstructor()) {
            return getJavaClass().getName();
        }
        return getElemType(elem);
    }

    public String getTypeFqn() {
        if (isConstructor()) {
            return getJavaClass().getFqn();
        }
        return getElemTypeFqn(elem);
    }

    public String getElemId() {
        return getElemId(elem);
    }

    public JavaClass getJavaClass() {
        return getJavaClass(elem);
    }

    public ArrayList getParameterNodes() {
        return getChildNodes(elem, JXConstants.ParameterElem);
    }

    public JavaParameter getParameterNode(String name) {
        Iterator it = getParameterNodes().iterator();
        while (it.hasNext()) {
            JavaParameter jp = new JavaParameter((Node)it.next());

            if (jp.getName().equals(name)) {
                return jp;
            }
        }
        return null;
    }

    public int getNumParameter() {
        return getParameterNodes().size();
    }

    public int getParameterIndex(JavaParameter jparam) {
        int num = 0;

        Iterator it = getParameterNodes().iterator();
        while (it.hasNext()) {
            JavaParameter jp = new JavaParameter((Node)it.next());
            if (jp.equals(jparam)) {
                return num;
            }
            num++;
        }
        return -1;
    }

    public String getParameterTypeFqn(int index) {
        if (index < getNumParameter()) {
            Node node = (Node)getParameterNodes().get(index);
            JavaParameter jp = new JavaParameter(node);
            return jp.getTypeFqn();
        }
        return null;
    }

    public ArrayList getLocalNodes() {
        return getDescendantNodes(elem, JXConstants.LocalElem);
    }

    public ArrayList getMethodCallNodes() {
        ArrayList mcalls = getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.MethodCall);
        mcalls.addAll(getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.ConstructorCall));
        mcalls.addAll(getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.SuperConstructorCall));
        return mcalls;
    }

    public ArrayList getVariableAccessNodes() {
        return getDescendantNodes(elem,
               JXConstants.ExpressionElem, JXConstants.VarRef);
    }

    public JavaVariableList getVariableList() {
        JavaVariableList vars = new JavaVariableList();

        vars.addAll(getReferredVariables());
        vars.addAll(getLocalVariables());
        vars.addAll(getParameterVariables());
        return vars;
    }

    private JavaVariableList getReferredVariables() {
        ArrayList nodes = getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.VarRef);
        JavaVariableList vars = new JavaVariableList();

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            Node node = (Node)it.next();

            JavaVariable jv = new JavaVariable(node);
            vars.add(jv);
        }
        return vars;
    }

    private JavaVariableList getLocalVariables() {
        ArrayList nodes = getDescendantNodes(elem, JXConstants.LocalElem);
        JavaVariableList vars = new JavaVariableList();

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            Node node = (Node)it.next();

            JavaVariable jv = new JavaVariable(node);
            vars.add(jv);
        }
        return vars;
    }

    private JavaVariableList getParameterVariables() {
        ArrayList nodes = getDescendantNodes(elem, JXConstants.ParameterElem);
        JavaVariableList vars = new JavaVariableList();

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            Node node = (Node)it.next();

            JavaVariable jv = new JavaVariable(node);
            vars.add(jv);
        }
        return vars;
    }

    public JavaVariableList getDefVariableList() {
        JavaVariableList vars = new JavaVariableList();

        Iterator it = getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            if (jv.isDefVariable()) {
                vars.add(jv);
            }
        }
        return vars;
    }

    public JavaVariableList getUseVariableList() {
        JavaVariableList vars = new JavaVariableList();

        Iterator it = getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            if (jv.isUseVariable()) {
                vars.add(jv);
            }
        }
        return vars;
    }

    public JavaClassList getUsedTypes() {
        JavaClassList classes = new JavaClassList();

        Iterator it = getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaType jt = new JavaType(elem);
            JavaClass jc = jt.getReferredJavaClass();
            if (jc != null) {
                classes.add(jc);
            }
        }
        return classes;
    }

    public ArrayList getStatementNodes() {
        ArrayList statements = getDescendantNodes(getElem(), JXConstants.StatementElem);
        statements.remove(0);
        return statements;
    }

    public boolean equals(JavaMethod jm) {
        return jm != null && isInTheSameClass(jm) &&
            getSignature().equals(jm.getSignature());
    }

    public ArrayList getThrowsNodes() {
        return getChildNodes(elem, JXConstants.ThrowsElem);
    }

    public ArrayList getThrowsTypes() {
        ArrayList types = new ArrayList();

        Iterator it = getThrowsNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaElement je = new JavaElement(elem);
            String type = je.getElemType(elem);
            types.add(type);
        }
        return types;
    }

    public JavaClassList getAllThrows() {
        JavaClassList jreceivers = new JavaClassList();

        Iterator it = getThrowsNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();
            JavaElement je = new JavaElement(elem);

            String fqn = je.getElemTypeFqn(elem);
            JavaClass jc = Repository.getJavaClass(fqn, getJavaFile());
            if (jc != null) {
                jreceivers.add(jc);
            }
        }
        return jreceivers;
    }

    public boolean isThrows() {
        Node node = getFirstChildNode(elem, JXConstants.ThrowsElem);
        return node.getNodeName().length() != 0;
    }

    public boolean isConstructor() {
        return elem.getNodeName().equals(JXConstants.ConstructorElem);
    }

    public Node getLastParameterNode() {
        ArrayList parameters = getParameterNodes();
        if (parameters.size() > 0) {
            return (Node)parameters.get(parameters.size() - 1);
        }
        return null;
    }

    public String getAccess() {
        String access = elem.getAttribute(JXConstants.AccessAttr);
        if (access.length() == 0) {
            access = JXConstants.Default;
        }
        return access;
    }

    public boolean isVoid() {
        return getType().equals("void");
    }

    public boolean isPublic() {
        if (getJavaClass() != null && getJavaClass().isInterface()) {
            return true;
        }
        return getAccess().equals(JXConstants.Public);
    }

    public boolean isProtected() {
        return getAccess().equals(JXConstants.Protected);
    }

    public boolean isPrivate() {
        return getAccess().equals(JXConstants.Private);
    }

    public boolean isDefault() {
        return getAccess().equals(JXConstants.Default);
    }

    public boolean isFinal() {
        return elem.getAttribute(JXConstants.FinalAttr).equals(JXConstants.YES);
    }

    public boolean isAbstract() {
        if (getJavaClass() != null && getJavaClass().isInterface()) {        
            return true;
        }
        return elem.getAttribute(JXConstants.AbstractAttr).equals(JXConstants.YES);
    }

    public boolean isStatic() {
        return elem.getAttribute(JXConstants.StaticAttr).equals(JXConstants.YES);
    }

    public boolean isSynchronized() {
        return elem.getAttribute(JXConstants.SynchronizedAttr).equals(JXConstants.YES);
    }

    public boolean isNative() {
        return elem.getAttribute(JXConstants.NativeAttr).equals(JXConstants.YES);
    }

    public boolean isStrictfp() {
        return elem.getAttribute(JXConstants.StrictfpAttr).equals(JXConstants.YES);
    }

    public String getModifier() {
        StringBuffer buf = new StringBuffer();

        if (isPublic()) {
            buf.append(" public");
        } else if (isProtected()) {
            buf.append(" protected");
        } else if (isPrivate()) {
            buf.append(" private");
        }

        if (isFinal()) {
            buf.append(" final");
        }
        if (isAbstract()) {
            buf.append(" abstract");
        }
        if (isStatic()) {
            buf.append(" static");
        }
        if (isSynchronized()) {
            buf.append(" synchronized");
        }
        if (isNative()) {
            buf.append(" native");
        }
        if (isStrictfp()) {
            buf.append(" strictfp");
        }

        if (buf.length() != 0) {
            return buf.substring(1);
        }
        return "";
    }

    public String getSignature() {
        if (isConstructor()) {
            return getJavaClass().getName() + "(" + getParameterTypes() + " )";
        }
        return getName() + "(" + getParameterTypes() + " )";
    }

    public String getSignatureWithTheName(String name) {
        return name + "(" + getParameterTypes() + " )";
    }

    private String getParameterTypes() {
        StringBuffer buf = new StringBuffer();

        Iterator it = getParameterNodes().iterator();
        while (it.hasNext()) {
            JavaParameter jp = new JavaParameter((Node)it.next());

            buf.append(" ");
            buf.append(jp.getTypeFqn());
        }
        return buf.toString();
    }

    public JavaClassList getAllInnerClassesAndInterfaces() {
        JavaClassList jclasses = new JavaClassList();

        Iterator it = getDescendantNodes(getElem(), JXConstants.ClassElem).iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jclasses.add(jc);
        }

        it = getDescendantNodes(getElem(), JXConstants.InterfaceElem).iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jclasses.add(jc);
        }
        return jclasses;
    }

    public Node getIdentNode() {
        return getFirstChildNode(elem, JXConstants.IdentElem);
    }

    public void print(StringBuffer buf) {
        print("", buf);
    }

    public void print(String indent, StringBuffer buf) {
        buf.append(indent + "METHOD: " + getAccess() + " " + getName());
        buf.append("@" + getType() + "[" + getTypeFqn() + "]\n");
        printAllParameters(indent, buf);
        // printAllMethodCalls(indent, buf);
        // printAllVarRefs(indent, buf);
    }

    public void printAllParameters(String indent, StringBuffer buf) {
        Iterator it = getParameterNodes().iterator();
        while (it.hasNext()) {
            JavaParameter jp = new JavaParameter((Node)it.next());
            jp.print(indent + "  ", buf);
        }
    }

    /*
    public void printAllMethodCalls(String indent, StringBuffer buf) {
        Iterator it = getExpressionNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            
            if (getElemSort(e).equals(JXConstants.MethodCall)) {
                JavaCall ja = new JavaCall(e);
                ja.print(indent, buf);
            }
        }
    }
    */
    
    /*
    public void printAllVarRefs(String indent, StringBuffer buf) {
        Iterator it = getExpressionNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            
            if (getElemSort(e).equals(JXConstants.VarRef)) {
                JavaVariable jv = new JavaVariable(e);
                jv.print(buf);
            }
        }
    }
    */
}
