/*
 *     JavaStaticInit.java  Apr 19, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.j2xml.Repository;
import org.w3c.dom.*;
import java.util.*;

public class JavaStaticInit extends JavaElement {

    protected JavaStaticInit() {
    }        

    public JavaStaticInit(Node node) {
        super(node);
    }

    public String getElemId() {
        return getElemId(elem);
    }

    public JavaClass getJavaClass() {
        return getJavaClass(elem);
    }

    public ArrayList getLocalNodes() {
        return getDescendantNodes(elem, JXConstants.LocalElem);
    }

    public ArrayList getMethodCallNodes() {
        return getDescendantNodes(elem,
               JXConstants.ExpressionElem, JXConstants.MethodCall);
    }

    public ArrayList getVariableAccessNodes() {
        return getDescendantNodes(elem,
               JXConstants.ExpressionElem, JXConstants.VarRef);
    }

    public JavaVariableList getVariableList() {
        JavaVariableList vars = new JavaVariableList();

        vars.addAll(getReferredVariables());
        vars.addAll(getLocalVariables());
        return vars;
    }

    private JavaVariableList getReferredVariables() {
        ArrayList nodes = getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.VarRef);
        JavaVariableList vars = new JavaVariableList();

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            Node node = (Node)it.next();

            JavaVariable jv = new JavaVariable(node);
            vars.add(jv);
        }
        return vars;
    }

    private JavaVariableList getLocalVariables() {
        ArrayList nodes = getDescendantNodes(elem, JXConstants.LocalElem);
        JavaVariableList vars = new JavaVariableList();

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            Node node = (Node)it.next();

            JavaVariable jv = new JavaVariable(node);
            vars.add(jv);
        }
        return vars;
    }

    public JavaVariableList getDefVariableList() {
        JavaVariableList vars = new JavaVariableList();

        Iterator it = getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            if (jv.isDefVariable()) {
                vars.add(jv);
            }
        }
        return vars;
    }

    public JavaVariableList getUseVariableList() {
        JavaVariableList vars = new JavaVariableList();

        Iterator it = getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            if (jv.isUseVariable()) {
                vars.add(jv);
            }
        }
        return vars;
    }

    public JavaClassList getUsedTypes() {
        JavaClassList classes = new JavaClassList();

        Iterator it = getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaType jt = new JavaType(elem);
            JavaClass jc = jt.getReferredJavaClass();
            if (jc != null) {
                classes.add(jc);
            }
        }
        return classes;
    }

    public boolean equals(JavaStaticInit js) {
        return js != null && isInTheSameClass(js); 
    }

    public boolean isStatic() {
        //return elem.getAttribute(JXConstants.StaticAttr).equals(JXConstants.YES);
        return elem.getFirstChild().getNodeName().equals(JXConstants.KeywordElem);
    }
}
