/*
 *     JavaVariable.java  Apr 19, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.j2xml.Repository;
import org.w3c.dom.*;

/*
  An object of this class will be never cached.
*/
public class JavaVariable extends JavaExpr {
    private Element decl = null;  // for only intra-file
    private JavaField refField = null;

    protected JavaVariable() {
    }

    public JavaVariable(Node node, Element decl) {
        super(node);
        this.decl = decl;
    }

    public JavaVariable(Node node) {
        this(node, null);

        String defid = getId();
        if (defid.length() != 0) {

            JavaMethod jmethod = getJavaMethod();
            Node topNode;
            if (jmethod != null) {
                topNode = jmethod.getElem();
                while (true) {
                    Node n = getMethodOrConstructorNode(topNode.getParentNode());
                    if (n == null) {
                        break;
                    }
                    topNode = n;
                }
            } else {
                topNode = elem;
                while (topNode != null && !topNode.getNodeName().equals(JXConstants.StaticInitElem)) {
                    topNode = topNode.getParentNode();
                }            
            }

            if (topNode != null) {
                if (decl == null) {
                    NodeList nl = ((Element)topNode).getElementsByTagName(JXConstants.LocalElem);
                    decl = getElemFromId(nl, defid);
                }

                if (decl == null) {
                    NodeList nl = ((Element)topNode).getElementsByTagName(JXConstants.ParameterElem);
                    decl = getElemFromId(nl, defid);
                }
            }

            if (decl == null){
                String ref = getRefClass();
                JavaFile jf;
                JavaClass refClass;

                if (ref.length() == 0) {
                    refClass = getJavaClass();
                    jf = refClass.getJavaFile();

                } else {
                    int sep = ref.indexOf("$");
                    if (sep != -1) {
                        ref = ref.substring(0, sep);
                    }
                    jf = getJavaFile();
                    refClass = Repository.getJavaClass(ref, jf);
                    
                    if (refClass != null && jf.getName().equals(refClass.getJavaFile().getName())) {
                        NodeList nl = jf.getElem().getElementsByTagName(JXConstants.FieldElem);
                        decl = getElemFromId(nl, defid);
                    }
                }
            }
        }
    }

    public String getId() {
        Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
        return getElemId(e);
    }

    public String getElemId() {
        return getElemId(elem);
    }

    public String getName() {
        return getIdentName(elem);
    }

    public JavaClass getJavaClass() {
        return getJavaClass(elem);
    }

    public JavaMethod getJavaMethod() {
        return getJavaMethod(elem);
    }

    public Node getStatementNode() {
        Node node = getElem();
        while (node != null && !isStatementNode(node) &&
               !isFieldNode(node) && !isLocalNode(node) && !isParameterNode(node)) {
            node = node.getParentNode();
        }
        return node;
    }

    public String getCastType() {
        Element parent = (Element)elem.getParentNode();
        if (getElemSort(parent).equals(JXConstants.Cast)) {

            Element exp = (Element)getFirstChildNode(parent, JXConstants.ExpressionElem);
            if (getElemSort(exp).equals(JXConstants.TypeRef)) {

                Element e = (Element)getFirstChildNode(exp, JXConstants.TypeElem);
                JavaType jt = new JavaType(e);
                String fqn = jt.getFqn();
                if (fqn.length() == 0) {
                    fqn = jt.getName();
                }
                return fqn;
            }
        }
        return null;
    }
    
    public String getType() {
        if (decl != null) {
            return getElemType(decl);
        }
        return getTypeFqn();
    }

    public String getTypeFqn() {
        String cast = getCastType();
        if (cast != null) {
            return cast;
        }

        if (decl != null) {
            return getElemTypeFqn(decl);

        } else {
            Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
            return e.getAttribute(JXConstants.FqnAttr);
        }
    }

    public Node getDeclaration() {
            return decl;
    }

    public boolean isDeclarationVariable() {
        if (decl != null) {
            JavaVariable jv = null;

            if (decl.getNodeName().equals(JXConstants.FieldElem)) {
                JavaField jf = new JavaField(decl);
                jv = jf.getJavaVariable();
            }
            if (decl.getNodeName().equals(JXConstants.LocalElem)) {
                JavaLocal jl = new JavaLocal(decl);
                jv = jl.getJavaVariable();
            }
            if (decl.getNodeName().equals(JXConstants.ParameterElem)) {
                JavaParameter jp = new JavaParameter(decl);
                jv = jp.getJavaVariable();
            }

            if (jv != null && strictlyEquals(jv)) {
                return true;
            }
        }
        return false;
    }

    public String getRefClass() {
        Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
        return e.getAttribute(JXConstants.RefAttr);
    }

    public String getRefFieldId() {
        Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
        return e.getAttribute(JXConstants.DefIdAttr);
    }

    public JavaField getUsedJavaField() {        
        if (refField == null && isField()) {
            JavaClass jc = Repository.getJavaClass(getRefClass(), getJavaFile());
            String defid = getRefFieldId();
            
            if (jc != null && defid.length() != 0) {
                Node n = jc.getFieldNode(defid);
                refField = new JavaField(n);
            }
        }
        return refField;
    }

    public boolean equals(JavaVariable jv) {
        if (this == jv) {
            return true;
        }
        return jv != null && isInTheSameFile(jv) && getId().equals(jv.getId());
    }

    public boolean strictlyEquals(JavaVariable jv) {
        if (this == jv) {
            return true;
        }
        return jv != null && isInTheSameFile(jv) && getElemId().equals(jv.getElemId());
    }

    public boolean isPrimitive() {
        Element e = getTypeNode(elem);
        String sort = getElemSort(e);
        if (sort.equals(JXConstants.Primitive)) {
            return true;
        }
        return false;
    }

    public boolean isDefVariable() {
        return elem.getAttribute(JXConstants.WriteAttr).equals(JXConstants.YES);
    }

    public boolean isUseVariable() {
        return elem.getAttribute(JXConstants.ReadAttr).equals(JXConstants.YES);
    }

    public boolean isNonDefUseVariable() {
        return !isDefVariable() && !isUseVariable();
    }

    public boolean isField() {
        if (decl != null) {
            return isFieldNode(decl);
        }
        return true;
    }        

    public boolean isLocal() {
        if (decl != null) {
            return isLocalNode(decl);
        }
        return false;
    }        

    public boolean isParameter() {
        if (decl != null) {
            return isParameterNode(decl);
        }
        return false;
    }        

    public boolean isMethodReturn() {
        if (getName().startsWith("$")) {
            return true;
        }
        return false;
    }

    public Node getIdentNode() {
        return getFirstChildNode(elem, JXConstants.IdentElem);
    }

    public void print(StringBuffer buf) {
        buf.append(getName() + "@" + getTypeFqn() + " " + getId() + "\n");
    }
}
